/* 
 * $TSUKUBA_Release: Omni OpenMP/XcalableMP Compiler Software Version 0.6.0 (alpha) $
 * $TSUKUBA_Copyright:
 *  Copyright (C) 2010-2011 University of Tsukuba, 
 *  	      2012  University of Tsukuba and Riken AICS
 *  
 *  This software is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License version
 *  2.1 published by the Free Software Foundation.
 *  
 *  Please check the Copyright and License information in the files named
 *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
 *  Software release kit.
 *  
 *  * The specification of XcalableMP has been designed by the XcalableMP
 *    Specification Working Group (http://www.xcalablemp.org/).
 *  
 *  * The development of this software was partially supported by "Seamless and
 *    Highly-productive Parallel Programming Environment for
 *    High-performance computing" project funded by Ministry of Education,
 *    Culture, Sports, Science and Technology, Japan.
 *  $
 */
#include <stdlib.h>
#include <stdarg.h> 
#include <wchar.h>
#include "c-comp.h"
#include "c-option.h"

#include "c-xmp.h"
#include "c-xcodeml.h"

void outx_XMP_Clause(FILE *fp, int indent, CExprOfList* clause);
char *xmpDirectiveName(int c);

void
out_XMP_PRAGMA(FILE *fp, int indent, int pragma_code, CExpr* expr)
{
    CExprOfList *body = (CExprOfList *)expr;
    CExprOfList *clauseList = (CExprOfList *)body->e_aux_info;

    const char *ompPragmaTag = "XMPPragma";
    int indent1 = indent + 1;

    outxTagForStmt(fp, indent,(CExpr*)clauseList, ompPragmaTag,0, NULL);
    outxPrint(fp,indent1,"<string>%s</string>\n",
	      xmpDirectiveName(pragma_code));
    outx_XMP_Clause(fp,indent1,clauseList);
    if(EXPR_L_SIZE(expr) != 0) outxChildren(fp,indent1,expr); // body
    outxTagClose(fp, indent,ompPragmaTag);
}

void outx_XMP_Clause(FILE *fp, int indent, CExprOfList* clauseList)
{
    int indent1 = indent + 1;
    CCOL_DListNode *ite;

    outxPrint(fp,indent1,"<list>\n");

    switch (clauseList->e_aux){

    case XMP_DIST_DUPLICATION:
      break;

    case XMP_DIST_BLOCK:
      outxPrint(fp,indent1+1,"<string>block</string>\n");
      break;

    case XMP_DIST_CYCLIC:
      outxPrint(fp,indent1+1,"<string>cyclic</string>\n");
      break;

    case XMP_DIST_GBLOCK:
      outxPrint(fp,indent1+1,"<string>gblock</string>\n");
      break;
    }

    EXPR_FOREACH(ite, clauseList){
	CExpr *node = EXPR_L_DATA(ite);
	//	if(node == NULL) 
	if(EXPR_ISNULL(node)) 
	    outxPrint(fp,indent1+1,"<list/>\n");
	else if(EXPR_CODE(node) == EC_UNDEF)
	    outx_XMP_Clause(fp,indent1,(CExprOfList *)node);
	else
	    outxContext(fp,indent1+1,node);
    }
    outxPrint(fp,indent1,"</list>\n");
}

char *xmpDirectiveName(int c)
{
  switch(c){
  case XMP_NODES: return "NODES";
  case XMP_TEMPLATE: return "TEMPLATE";
  case XMP_DISTRIBUTE: return "DISTRIBUTE";
  case XMP_ALIGN: return "ALIGN";
  case XMP_SHADOW: return "SHADOW";
  case XMP_TASK: return "TASK";
  case XMP_TASKS: return "TASKS";
  case XMP_LOOP: return "LOOP";
  case XMP_REFLECT: return "REFLECT";
  case XMP_GMOVE: return "GMOVE";
  case XMP_BARRIER: return "BARRIER";
  case XMP_REDUCTION: return "REDUCTION";
  case XMP_BCAST: return "BCAST";
  case XMP_COARRAY: return "COARRAY";
  case XMP_TEMPLATE_FIX: return "TEMPLATE_FIX";
  default: return "OMP???";
  }
}
